@@ -8,13 +8,20 @@
 -- Further documentation and information can be found at
 -- <http://www.haskell.org/haskellwiki/Library/Data_encoding>.
 module Codec.Binary.Base16
-    ( encode
+    ( EncIncData(..)
+    , EncIncRes(..)
+    , encodeInc
+    , encode
+    , DecIncData(..)
+    , DecIncRes(..)
+    , decodeInc
     , decode
-    , decode'
     , chop
     , unchop
     ) where
 
+import Codec.Binary.Util
+
 import Control.Monad
 import Data.Array
 import Data.Bits
@@ -38,29 +45,42 @@ decodeMap :: M.Map Char Word8
 decodeMap  = M.fromList [(snd i, fst i) | i <- _encMap]
 
 -- {{{1 encode
+-- | Incremental encoder function.
+encodeInc :: EncIncData -> EncIncRes String
+encodeInc EDone = EFinal []
+encodeInc (EChunk os) = EPart (concat $ map toHex os) encodeInc
+
 -- | Encode data.
-encode :: [Word8]
-    -> String
-encode os = let
-        splitOctet o = [ o `shiftR` 4, o .&. 0xf ]
-    in map (encodeArray !) $ foldr ((++) . splitOctet) [] os
+encode :: [Word8] -> String
+encode = encoder encodeInc
 
 -- {{{1 decode
--- | Decode data (lazy).
-decode' :: String
-    -> [Maybe Word8]
-decode' = let
-        dec [] = []
-        dec (Just o1 : Just o2 : os) =
-                    Just (o1 `shiftL` 4 .|. o2) : dec os
-        dec _ = [Nothing]
-    in
-        dec . map (flip M.lookup decodeMap)
+-- | Incremental decoder function.
+decodeInc :: DecIncData String -> DecIncRes String
+decodeInc d = dI [] d
+    where
+        dec2 cs = let
+                ds = map (flip M.lookup decodeMap) cs
+                es@[e1, e2] = map fromJust ds
+                o = e1 `shiftL` 4 .|. e2
+                allJust = and . map isJust
+            in if allJust ds
+                then Just o
+                else Nothing
 
--- | Decode data (strict).
-decode :: String
-    -> Maybe [Word8]
-decode = sequence . decode'
+        dI [] DDone = DFinal [] []
+        dI lo DDone = DFail [] lo
+        dI lo (DChunk s) = doDec [] (lo ++ s)
+            where
+                doDec acc s'@(c1:c2:cs) = maybe
+                    (DFail acc s')
+                    (\ b -> doDec (acc ++ [b]) cs)
+                    (dec2 [c1, c2])
+                doDec acc s = DPart acc (dI s)
+
+-- | Decode data.
+decode :: String -> Maybe [Word8]
+decode = decoder decodeInc
 
 -- {{{1 chop
 -- | Chop up a string in parts.
