@@ -9,13 +9,20 @@
 -- Further documentation and information can be found at
 -- <http://www.haskell.org/haskellwiki/Library/Data_encoding>.
 module Codec.Binary.Base64
-    ( encode
+    ( EncIncData(..)
+    , EncIncRes(..)
+    , encodeInc
+    , encode
+    , DecIncData(..)
+    , DecIncRes(..)
+    , decodeInc
     , decode
-    , decode'
     , chop
     , unchop
     ) where
 
+import Codec.Binary.Util
+
 import Control.Monad
 import Data.Array
 import Data.Bits
@@ -48,44 +55,69 @@ decodeMap :: M.Map Char Word8
 decodeMap  = M.fromList [(snd i, fst i) | i <- _encMap]
 
 -- {{{1 encode
--- | Encode data.
-encode :: [Word8]
-    -> String
-encode = let
-        pad n = replicate n 0
-        enc [] = ""
-        enc l@[o] = (++ "==") . take 2 .enc $ l ++ pad 2
-        enc l@[o1, o2] = (++ "=") . take 3 . enc $ l ++ pad 1
-        enc (o1 : o2 : o3 : os) = let
+-- | Incremental encoder function.
+encodeInc :: EncIncData -> EncIncRes String
+encodeInc e = eI [] e
+    where
+        enc3 [o1, o2, o3] = cs
+            where
                 i1 = o1 `shiftR` 2
                 i2 = (o1 `shiftL` 4 .|. o2 `shiftR` 4) .&. 0x3f
                 i3 = (o2 `shiftL` 2 .|. o3 `shiftR` 6) .&. 0x3f
                 i4 = o3 .&. 0x3f
-            in foldr ((:) . (encodeArray !)) "" [i1, i2, i3, i4] ++ enc os
-    in enc
+                cs = map (encodeArray !) [i1, i2, i3, i4]
+
+        eI [] EDone = EFinal []
+        eI [o1] EDone = EFinal (take 2 cs ++ "==")
+            where cs = enc3 [o1, 0, 0]
+        eI [o1, o2] EDone = EFinal (take 3 cs ++ "=")
+            where cs = enc3 [o1, o2, 0]
+        eI lo (EChunk bs) = doEnc [] (lo ++ bs)
+            where
+                doEnc acc (o1:o2:o3:os) = doEnc (acc ++ enc3 [o1, o2, o3]) os
+                doEnc acc os = EPart acc (eI os)
+
+-- | Encode data.
+encode :: [Word8] -> String
+encode = encoder encodeInc
 
 -- {{{1 decode
--- | Decode data (lazy).
-decode' :: String
-    -> [Maybe Word8]
-decode' = let
-        pad n = replicate n $ Just 0
-        dec [] = []
-        dec l@[Just eo1, Just eo2] = take 1 . dec $ l ++ pad 2
-        dec l@[Just eo1, Just eo2, Just eo3] = take 2 . dec $ l ++ pad 1
-        dec (Just eo1 : Just eo2 : Just eo3 : Just eo4 : eos) = let
-                o1 = eo1 `shiftL` 2 .|. eo2 `shiftR` 4
-                o2 = eo2 `shiftL` 4 .|. eo3 `shiftR` 2
-                o3 = eo3 `shiftL` 6 .|. eo4
-            in Just o1 : Just o2 : Just o3 : dec eos
-        dec _ = [Nothing]
-    in
-        dec . map (flip M.lookup decodeMap) . takeWhile (/= '=')
+-- | Incremental decoder function.
+decodeInc :: DecIncData String -> DecIncRes String
+decodeInc d = dI [] d
+    where
+        dec4 cs = let
+                ds = map (flip M.lookup decodeMap) cs
+                es@[e1, e2, e3, e4] = map fromJust ds
+                o1 =  e1 `shiftL` 2 .|.  e2 `shiftR` 4
+                o2 =  e2 `shiftL` 4 .|.  e3 `shiftR` 2
+                o3 =  e3 `shiftL` 6 .|.  e4
+                allJust = and . map isJust
+            in if allJust ds
+                then Just [o1, o2, o3]
+                else Nothing
 
--- | Decode data (strict).
-decode :: String
-    -> Maybe [Word8]
-decode = sequence . decode'
+        dI [] DDone = DFinal [] []
+        dI lo DDone = DFail [] lo
+        dI lo (DChunk s) = doDec [] (lo ++ s)
+            where
+                doDec acc s@(c1:c2:'=':'=':cs) = maybe
+                    (DFail acc s)
+                    (\ bs -> DFinal (acc ++ take 1 bs) cs)
+                    (dec4 [c1, c2, 'A', 'A'])
+                doDec acc s@(c1:c2:c3:'=':cs) = maybe
+                    (DFail acc s)
+                    (\ bs -> DFinal (acc ++ take 2 bs) cs)
+                    (dec4 [c1, c2, c3, 'A'])
+                doDec acc s@(c1:c2:c3:c4:cs) = maybe
+                    (DFail acc s)
+                    (\ bs -> doDec (acc ++ bs) cs)
+                    (dec4 [c1, c2, c3, c4])
+                doDec acc s = DPart acc (dI s)
+
+-- | Decode data.
+decode :: String -> Maybe [Word8]
+decode = decoder decodeInc
 
 -- {{{1 chop
 -- | Chop up a string in parts.
