@@ -10,19 +10,26 @@
 -- <http://www.haskell.org/haskellwiki/Library/Data_encoding>.
 
 module Codec.Binary.Url
-    ( encode
+    ( EncIncData(..)
+    , EncIncRes(..)
+    , encodeInc
+    , encode
+    , DecIncData
+    , DecIncRes
+    , decodeInc
     , decode
-    , decode'
     , chop
     , unchop
     ) where
 
+import Codec.Binary.Util
+
 import qualified Data.Map as M
 import Data.Char(ord)
 import Data.Word(Word8)
+import Data.Maybe(isJust, fromJust)
 
-import Codec.Binary.Util(toHex,fromHex)
-
+-- {{{1 enc/dec map
 _unreservedChars = zip [65..90] "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
         ++ zip [97..122] "abcdefghijklmnopqrstuvwxyz"
         ++ zip [48..57] "0123456789"
@@ -35,28 +42,45 @@ decodeMap :: M.Map Char Word8
 decodeMap = M.fromList [(b, a) | (a, b) <- _unreservedChars]
 
 -- {{{1 encode
+-- | Incremental decoder function.
+encodeInc :: EncIncData -> EncIncRes String
+encodeInc e = eI e
+    where
+        enc [] = []
+        enc (o : os) = case (M.lookup o encodeMap) of
+            Just c -> c : enc os
+            Nothing -> ('%' : toHex o) ++ enc os
+
+        eI EDone = EFinal []
+        eI (EChunk bs) = EPart (enc bs) encodeInc
+
 -- | Encode data.
-encode :: [Word8]
-    -> String
-encode [] = ""
-encode (o : os) = case (M.lookup o encodeMap) of
-    Just c -> c : encode os
-    Nothing -> ('%' : toHex o) ++ encode os
+encode :: [Word8] -> String
+encode = encoder encodeInc
 
 -- {{{1 decode
--- | Decode data (lazy).
-decode' :: String
-    -> [Maybe Word8]
-decode' [] = []
-decode' ('%' : c0 : c1 : cs) = fromHex [c0, c1] : decode' cs
-decode' (c : cs)
-    | c /= '%' = (Just $ fromIntegral $ ord c) : decode' cs
-    | otherwise = [Nothing]
+-- | Incremental decoder function.
+decodeInc :: DecIncData String -> DecIncRes String
+decodeInc d = dI [] d
+    where
+        dI [] DDone = DFinal [] []
+        dI lo DDone = DFail [] lo
+        dI lo (DChunk s) = doDec [] (lo ++ s)
+            where
+                doDec acc [] = DPart acc (dI [])
+                doDec acc s'@('%':c0:c1:cs) = let
+                        o = fromHex [c0, c1]
+                    in if isJust o
+                        then doDec (acc ++ [fromJust o]) cs
+                        else DFail acc s'
+                doDec acc s'@(c:cs)
+                    | c /= '%' = doDec (acc ++ [fromIntegral $ ord c]) cs
+                    | otherwise = DPart acc (dI s')
 
--- | Decode data (strict).
+-- | Decode data.
 decode :: String
     -> Maybe [Word8]
-decode = sequence . decode'
+decode = decoder decodeInc
 
 -- {{{1 chop
 -- | Chop up a string in parts.
